/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.legato.testing.script;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import net.disy.legato.net.protocol.classpath.ClassPathSocketFactory;
import net.disy.legato.net.protocol.classpath.Handler;
import net.sourceforge.htmlunit.corejs.javascript.Scriptable;

import org.apache.commons.io.IOUtils;

import com.gargoylesoftware.htmlunit.BrowserVersion;
import com.gargoylesoftware.htmlunit.StringWebResponse;
import com.gargoylesoftware.htmlunit.WebClient;
import com.gargoylesoftware.htmlunit.WebResponse;
import com.gargoylesoftware.htmlunit.WebWindow;
import com.gargoylesoftware.htmlunit.html.HTMLParser;
import com.gargoylesoftware.htmlunit.html.HtmlPage;
import com.gargoylesoftware.htmlunit.javascript.host.Window;

public abstract class AbstractHtmlUnitScriptTest extends
		AbstractRhinoScriptTest {

	@Override
	public void before() throws Exception {
		super.before();
		Handler.register();
		ClassPathSocketFactory.register();
	}

	protected URL getPageURL() {
		final String pageResourceName = getPageResourceName();
		try {
			return new URL("classpath://resources" + pageResourceName);
		} catch (final MalformedURLException murlex) {
			throw new AssertionError(murlex);
		}
	}

	public String getPageResourceName() {
		return getPageResourceName(getClass());
	}

	protected String getPageResourceName(final Class<?> _class) {
		final String pageResourceName = "/"
				+ _class.getName().replace('.', '/') + ".xhtml";
		if (_class.getResource(pageResourceName) != null) {
			return pageResourceName;
		} else {
			if (_class.getSuperclass() != null) {
				return getPageResourceName(_class.getSuperclass());
			} else {
				throw new AssertionError("Could not find page resource.");
			}
		}
	}

	@SuppressWarnings("deprecation")
	protected Window getWindow() {
		final WebClient webClient = new WebClient(getBrowserVersion());

		final WebWindow webWindow = webClient.openWindow(null, "main");
		final Window window = (Window) webWindow.getScriptObject();

		final WebResponse webResponse = new StringWebResponse(getPageString(),
				getPageURL());

		HtmlPage page = null;
		try {
			page = HTMLParser.parse(webResponse, webWindow);
		} catch (final IOException ioex) {
			throw new AssertionError(ioex);
		}
		window.setDomNode(page);
		window.jsxGet_document().setDomNode(page);

		return window;
	}

	@Override
	protected Scriptable getScope() {
		return getWindow();
	}

	@Override
	protected Map<String, Object> getImplicitObjects() {
		final Map<String, Object> map = new HashMap<String, Object>(
				super.getImplicitObjects());
		/*
		 * final WebClient webClient = new WebClient(getBrowserVersion());
		 * 
		 * final WebWindow webWindow = webClient.openWindow(null, "main"); final
		 * Window window = (Window) webWindow.getScriptObject();
		 * 
		 * final WebResponse webResponse = new
		 * StringWebResponse(getPageString(), getPageURL());
		 * 
		 * HtmlPage page = null; try { page = HTMLParser.parse(webResponse,
		 * webWindow); } catch (final IOException ioex) { throw new
		 * AssertionError(ioex); } window.setDomNode(page);
		 * window.jsxGet_document().setDomNode(page);
		 * 
		 * map.put("window", window); map.put("document",
		 * window.jsxGet_document()); map.put("navigator",
		 * window.jsxGet_navigator()); map.put("top", window.jsxGet_top());
		 */
		map.put("headless", true);
		return map;
	}

	public BrowserVersion getBrowserVersion() {
		return BrowserVersion.INTERNET_EXPLORER_7;
	}

	protected String getPageString() throws AssertionError {
		InputStream is = null;
		try {
			final String pageResourceName = getPageResourceName();
			is = getClass().getResourceAsStream(pageResourceName);
			if (is == null) {
				throw new IOException("Could not load page resource ["
						+ pageResourceName + "].");
			}
			// text
			return IOUtils.toString(is, "UTF-8");
		} catch (final IOException ioex) {
			throw new AssertionError(ioex);
		} finally {
			IOUtils.closeQuietly(is);
		}
	}

}
