/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Copyright (c) 2008 by disy Informationssysteme GmbH
package net.disy.legato.testing.server;

import javax.servlet.ServletContext;

// NOT_PUBLISHED
/**
 * This interface models web server (like Jetty or Tomcat).
 */
public interface IWebServerEnvironment {

  /**
   * Returns the host under which this server runs.
   * @return Server host.
   */
  public String getHost();

  /**
   * Retunr the port under which this server runs.
   * @return Server port.
   */
  public int getPort();

  /**
   * Returns the base URL (<code>http://server:port/</code>).
   * @return Base URL.
   */
  public String getBaseUrl();

  /**
   * Checks if the server is started.
   * @return <code>true</code> if the server is started, <code>false</code> otherwise.
   */
  public boolean isStarted();

  /**
   * Checks if the server is stopped.
   * @return <code>true</code> if the server is stopped, <code>false</code> otherwise.
   */
  public boolean isStopped();

  /**
   * Starts the server.
   * @throws Exception In case there's a problem starting the server.
   */
  public void start() throws Exception;

  /**
   * Stops the server.
   * @throws Exception In case there's a problem stopping.
   */
  public void stop() throws Exception;

  /**
   * Joins the server thread with current thread.
   * @throws Exception In case there's a problem joining threads.
   */
  public void join() throws Exception;

  /**
   * Adds a web application context to this server.
   * @param contextPath context path (like <code>/myApp</code>).
   * @param location location - directory containing the <code>WEB-INF</code> for <code>myApp</code>.
   */
  public void addWebAppContext(String contextPath, String location);

  /**
   * Adds a web application context to this server.
   * @param contextPath context path (like <code>/myApp</code>).
   * @param location location - directory containing the <code>WEB-INF</code> for <code>myApp</code>.
   * @param cookies - should cookies be used by this context for session tracking
   */
  public void addWebAppContext(String contextPath, String location, boolean cookies);

  /**
   * Adds a servlet with given mapping to the web application.
   * @param contextPath target context path (<code>/myApp</code>).
   * @param servletClassName class name of the servler.
   * @param servletMapping servlet mapping (like <code>/myServlet</code>, the servlet will be accessible under
   * <code>/myApp/myServler</code>).
   */
  public void addServletWithMapping(
      final String contextPath,
      final String servletClassName,
      final String servletMapping);

  /**
   * Returns servlet context for the given context path or <code>null</code> if
   * context with this path is not found.
   * @param contextPath context path.
   * @return Servlet context for the given context path.
   */
  public ServletContext getServletContext(String contextPath);

}
