/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Copyright (c) 2008 by disy Informationssysteme GmbH
package net.disy.legato.testing.server.jetty;

import java.util.Arrays;
import java.util.Collections;

import javax.servlet.ServletContext;

import net.disy.legato.testing.server.IWebServerEnvironment;

import org.apache.commons.lang.ObjectUtils;
import org.mortbay.jetty.Connector;
import org.mortbay.jetty.Handler;
import org.mortbay.jetty.Server;
import org.mortbay.jetty.handler.ContextHandler;
import org.mortbay.jetty.nio.SelectChannelConnector;
import org.mortbay.jetty.servlet.ServletHandler;
import org.mortbay.jetty.webapp.WebAppContext;

// NOT_PUBLISHED
public class JettyWebServerEnvironment implements IWebServerEnvironment {

  private static final String[] DEFAULT_CONFIGURATION_CLASSES = new String[]{
      "org.mortbay.jetty.webapp.WebInfConfiguration",
      "org.mortbay.jetty.webapp.WebXmlConfiguration",
      "org.mortbay.jetty.webapp.JettyWebXmlConfiguration",
  // "org.mortbay.jetty.webapp.TagLibConfiguration"
  };

  private final Server server;

  private final Connector connector;

  // private final WebAppContext webAppContext;

  public JettyWebServerEnvironment(final int port) {
    this.server = new Server();
    this.connector = new SelectChannelConnector();
    connector.setPort(port);
    this.server.setConnectors(new Connector[]{ connector });
    // this.webAppContext = new WebAppContext();
    // webAppContext.setContextPath(contextPath);
    // webAppContext.setConfigurationClasses(DEFAULT_CONFIGURATION_CLASSES);
    // webAppContext.setWar(location);
  }

  @Override
  public void join() throws Exception {
    server.join();
  }

  public void start() throws IllegalStateException, Exception {
    if (server.isStarted()) {
      // initializeApplicationContext();
    }
    else if (server.isStarting()) {
      waitUntilStarted();
      // initializeApplicationContext();
    }
    else if (server.isStopping()) {
      waitUntilStopped();
      doStart();
    }
    else if (server.isFailed()) {
      throw new IllegalStateException("Could not start a server which has failed."); //$NON-NLS-1$
    }
    else {
      doStart();
    }
  }

  private void doStart() throws Exception, AssertionError {
    server.start();
    waitUntilStarted();
    // initializeRequest();
    // initializeApplicationContext();
  }

  // private HttpServletRequest request;
  //
  // public HttpServletRequest getRequest() {
  // return request;
  // }

  // private void initializeRequest() {
  // if (getRequest() == null) {
  // request = new MockHttpServletRequest();
  // for (final EventListener eventListener :
  // webAppContext.getEventListeners()) {
  // if (eventListener instanceof ServletRequestListener) {
  // ((ServletRequestListener) eventListener).requestInitialized(new
  // ServletRequestEvent(
  // getServletContext(),
  // getRequest()));
  // }
  // }
  // }
  // }

  // private void destroyRequest() {
  // if (getRequest() != null) {
  // for (final EventListener eventListener :
  // webAppContext.getEventListeners()) {
  // if (eventListener instanceof ServletRequestListener) {
  // ((ServletRequestListener) eventListener).requestInitialized(new
  // ServletRequestEvent(
  // getServletContext(),
  // getRequest()));
  // }
  // }
  // }
  // }

  private void waitUntilStarted() {
    while (!server.isFailed() && !server.isStarted()) {
      try {
        Thread.sleep(1000);
      }
      catch (final InterruptedException ignored) {
        break;
      }
    }
    if (server.isFailed()) {
      throw new IllegalStateException("Server has failed.");
    }
  }

  private void waitUntilStopped() {
    while (!server.isFailed() && !server.isStopped()) {
      try {
        Thread.sleep(1000);
      }
      catch (final InterruptedException ignored) {
        break;
      }
    }
    if (server.isFailed()) {
      throw new IllegalStateException("Server has failed.");
    }
  }

  public void stop() throws Exception {
    if (server.isStopped()) {
      // Nothing to do
    }
    else if (server.isStarting()) {
      waitUntilStarted();
      doStop();
    }
    else if (server.isStarted()) {
      doStop();
    }
    else if (server.isStopping()) {
      waitUntilStopped();
    }
    else if (server.isFailed()) {
      throw new IllegalStateException("Could not stop a server which has failed.");
    }
    else {
      doStop();
    }
  }

  private void doStop() throws Exception {
    // destroyRequest();
    server.stop();
    waitUntilStopped();
  }

  public boolean isStarted() {
    return server.isStarted();
  }

  public boolean isStopped() {
    return server.isStopped();
  }

  // public String getContextPath() {
  // return webAppContext.getContextPath();
  // }

  @Override
  public void addWebAppContext(final String contextPath, final String location) {
    final WebAppContext webAppContext = new WebAppContext();
    webAppContext.setContextPath(contextPath);
    webAppContext.setConfigurationClasses(DEFAULT_CONFIGURATION_CLASSES);
    webAppContext.setWar(location);
    webAppContext
        .setDefaultsDescriptor("net/disy/legato/testing/server/jetty/webapp/webdefault.xml");
    // final ServletHolder servlet = webAppContext.getServletHandler()
    // .getServlet("default");
    // String aaa = servlet.getInitParameter("useFileMappedBuffer");
    // servlet.setInitParameter("useFileMappedBuffer", "true");
    server.addHandler(webAppContext);
  }

  @Override
  public void addWebAppContext(
      final String contextPath,
      final String location,
      final boolean cookies) {
    //Jetty does not support cookies setting per context
    addWebAppContext(contextPath, location);
  }

  public void addServletWithMapping(
      final String contextPath,
      final String servletClassName,
      final String servletMapping) {
    final ContextHandler contextHandler = new ContextHandler();
    contextHandler.setContextPath(contextPath);
    final ServletHandler servletHandler = new ServletHandler();
    servletHandler.addServletWithMapping(servletClassName, contextPath + servletMapping);
    contextHandler.setHandler(servletHandler);
    server.addHandler(servletHandler);
  }

  @Override
  public ServletContext getServletContext(final String contextPath) {
    final Iterable<Handler> handlers = (server.getHandlers() == null ? Collections
        .singletonList(server.getHandler()) : Arrays.asList(server.getHandlers()));
    for (final Handler handler : handlers) {
      if (handler instanceof ContextHandler) {
        final ContextHandler contextHandler = (ContextHandler) handler;
        if (ObjectUtils.equals(contextPath, contextHandler.getContextPath())) {
          return contextHandler.getServletContext();
        }
      }
    }
    return null;
  }

  // public void addServlet

  public String getHost() {
    return "127.0.0.1";
  }

  public int getPort() {
    return connector.getPort();
  }

  public String getBaseUrl() {
    return "http://" + getHost() + ':' + getPort();
  }

}
